import { useState, useEffect } from 'react';
import PropTypes from 'prop-types';
import { Seat } from '../Seat/Seat.jsx';
import styles from './SeatLayout.module.css';
import { seatsData } from './SeatsData.js';

const populateSeatsArray = (selectedSeats, reservedSeats, currentReservedSeats) => {
  return seatsData.map(({ row, seats }) =>
    seats.map((number) => {
      const seatNo = row + number;
      if (number === '/') return { seatNo, status: 'none' };

      const isMine = currentReservedSeats.includes(seatNo);
      const isSelected = selectedSeats.includes(seatNo);
      const isReserved = reservedSeats.includes(seatNo);

      if (isMine && isSelected) return { seatNo, status: 'selected' };
      if (isMine && !isSelected) return { seatNo, status: 'mine-unselected' };
      if (isReserved) return { seatNo, status: 'reserved' };
      if (isSelected) return { seatNo, status: 'selected' };

      return { seatNo, status: 'available' };
    })
  );
};

export const SeatLayout = ({
  selectedSeats: initialSelected = [],
  reservedSeats = [],
  currentReservedSeats = [],
  limit,
  onSelectionChange,
}) => {
  const [localSelectedSeats, setLocalSelectedSeats] = useState(currentReservedSeats);

  useEffect(() => {
    setLocalSelectedSeats(currentReservedSeats);
  }, [currentReservedSeats]);

  useEffect(() => {
    onSelectionChange?.(localSelectedSeats);
  }, [localSelectedSeats]);

  const seatClickHandler = (seat) => {
    const { seatNo, status } = seat;
    const isMine = currentReservedSeats.includes(seatNo);
    const isSelected = localSelectedSeats.includes(seatNo);

    if (status === 'reserved') return;

    setLocalSelectedSeats((prev) => {
      const updated = [...prev];

      if (isSelected) {
        return updated.filter((s) => s !== seatNo); // unselect
      } else {
    
        const newlySelected = updated.filter(
          (s) => !currentReservedSeats.includes(s)
        );

        const unselectedMine = currentReservedSeats.filter(
          (s) => !updated.includes(s)
        );

        const totalAllowed = limit + unselectedMine.length;

        if (newlySelected.length < totalAllowed) {
          return [...updated, seatNo];
        } else {
          return updated; // reject selection
        }
      }
    });
  };

  const renderedSeats = populateSeatsArray(
    localSelectedSeats,
    reservedSeats,
    currentReservedSeats
  );

  return (
    <div className={styles.container}>
      {renderedSeats.map((row, i) => (
        <div key={i} className={styles.seatcontainer}>
          {row.map((seat, j) => (
            <Seat key={j} {...seat} onClick={() => seatClickHandler(seat)} />
          ))}
        </div>
      ))}
    </div>
  );
};

SeatLayout.propTypes = {
  selectedSeats: PropTypes.arrayOf(PropTypes.string),
  reservedSeats: PropTypes.arrayOf(PropTypes.string),
  currentReservedSeats: PropTypes.arrayOf(PropTypes.string),
  limit: PropTypes.number.isRequired,
  onSelectionChange: PropTypes.func.isRequired,
};
